import rasterLayer from "./RasterLayer";
import layerEvents from "../../lib/layerEvents";

export default {
  name: "ArcgisLayer",
  mixins: [rasterLayer],
  props: {
    url: {
      type: String,
      require: true
    },
    tileSize: {
      type: Number,
      default: 256
    },
    zoomOffset: {
      type: Number,
      default: 1
    }
  },
  methods: {
    $_init() {
      if (this.url) {
        let url = this.url;
        if (url.indexOf("/{z}/{y}/{x}") === -1) {
          url += "/tile/" + "{z}" + "/" + "{y}" + "/" + "{x}" + ".jpg";
        }
        this._url = url;
      }
    },
    $_resetMapUrl() {
      let zoomOffset = this.zoomOffset;
      let _getQuadkey = function(z, x, y) {
        let quadkey = "",
          mask;
        for (let i = z; i > 0; i--) {
          mask = 1 << (i - 1);
          quadkey += (x && mask ? 1 : 0) + (y && mask ? 2 : 0);
        }
        return quadkey;
      };
      //重写mapbox中CanonicalTileID的url类，以支持非标准裁剪的瓦片（这里的非标准不是指任意裁剪瓦片，而是当前瓦片的0级不是mapbox定义的瓦片的0级，中间相差zoomOffset级）
      this.mapbox.CanonicalTileID.prototype.url = function(urls, scheme) {
        const bbox = this.getTileBBox();
        if (zoomOffset > this.z) {
          zoomOffset = 0;
        }
        const quadkey = _getQuadkey(this.z - zoomOffset, this.x, this.y);

        return urls[(this.x + this.y) % urls.length]
          .replace(
            "{prefix}",
            (this.x % 16).toString(16) + (this.y % 16).toString(16)
          )
          .replace("{z}", String(this.z - zoomOffset))
          .replace("{x}", String(this.x))
          .replace(
            "{y}",
            String(scheme === "tms" ? Math.pow(2, this.z) - this.y - 1 : this.y)
          );
      };
    },
    $_deferredMount() {
      this.$_init();
      this.$_resetMapUrl();
      let source = {
        type: "raster",
        tiles: [this._url],
        tileSize: this.tileSize,
        ...this.source
      };

      this.map.on("dataloading", this.$_watchSourceLoading);
      try {
        this.map.addSource(this.sourceId, source);
      } catch (err) {
        if (this.replaceSource) {
          this.map.removeSource(this.sourceId);
          this.map.addSource(this.sourceId, source);
        }
      }
      this.$_addLayer();
      this.$_bindLayerEvents(layerEvents);
      this.map.off("dataloading", this.$_watchSourceLoading);
      this.initial = false;
    }
  }
};
