import rasterLayer from "../RasterLayer";
import layerEvents from "../../../lib/layerEvents";
import igsOptions from "./igsOptions";

export default {
  name: "IgsTileLayer",
  mixins: [rasterLayer],
  props: {
    ...igsOptions,
    serverName: {
      type: String,
      require: true
    },
    zoomOffset: {
      type: Number,
      default: 0
    }
  },
  methods: {
    $_init() {
      if (this.url) {
        this._url = this.url;
        if (this.url.indexOf("{z}") < 0) {
          this._url += "/{z}/{y}/{x}";
        }
        return;
      }
      let domain = this.domain;
      if (!domain) {
        domain = this.protocol + "://" + this.ip + ":" + this.port;
      }
      this._url =
        encodeURI(domain + "/igs/rest/mrms/tile/" + this.serverName) +
        "/{z}/{y}/{x}";
    },
    $_resetMapUrl() {
      let zoomOffset = this.zoomOffset;
      let _getQuadkey = function(z, x, y) {
        let quadkey = "",
          mask;
        for (let i = z; i > 0; i--) {
          mask = 1 << (i - 1);
          quadkey += (x && mask ? 1 : 0) + (y && mask ? 2 : 0);
        }
        return quadkey;
      };
      //重写mapbox中CanonicalTileID的url类，以支持非标准裁剪的瓦片（这里的非标准不是指任意裁剪瓦片，而是当前瓦片的0级不是mapbox定义的瓦片的0级，中间相差zoomOffset级）
      this.mapbox.CanonicalTileID.prototype.url = function(urls, scheme) {
        const bbox = this.getTileBBox();
        if (zoomOffset > this.z) {
          zoomOffset = 0;
        }
        const quadkey = _getQuadkey(this.z - zoomOffset, this.x, this.y);

        return urls[(this.x + this.y) % urls.length]
          .replace(
            "{prefix}",
            (this.x % 16).toString(16) + (this.y % 16).toString(16)
          )
          .replace("{z}", String(this.z - zoomOffset))
          .replace("{x}", String(this.x))
          .replace(
            "{y}",
            String(scheme === "tms" ? Math.pow(2, this.z) - this.y - 1 : this.y)
          )
          .replace("{quadkey}", quadkey)
          .replace("{bbox}", bbox);
      };
    },
    $_deferredMount() {
      this.$_init();
      this.$_resetMapUrl();
      let source = {
        type: "raster",
        tiles: [this._url],
        tileSize: this.tileSize,
        ...this.source
      };

      this.map.on("dataloading", this.$_watchSourceLoading);
      try {
        this.map.addSource(this.sourceId, source);
      } catch (err) {
        if (this.replaceSource) {
          this.map.removeSource(this.sourceId);
          this.map.addSource(this.sourceId, source);
        }
      }
      this.$_addLayer();
      this.$_bindLayerEvents(layerEvents);
      this.map.off("dataloading", this.$_watchSourceLoading);
      this.initial = false;
    }
  }
};
