import rasterLayer from "../RasterLayer";
import layerEvents from "../../../lib/layerEvents";
import igsOptions from "./igsOptions";

export default {
  name: "IgsWmsLayer",
  mixins: [rasterLayer],
  props: {
    ...igsOptions,
    serverType: {
      type: String,
      require: true
    },
    serverName: {
      type: String,
      require: true
    },
    layers: {
      type: Array,
      require: true
    },
    version: {
      type: String,
      default: "1.1.1"
    },
    crs: {
      type: String,
      default: "EPSG:4326"
    },
    // styles: {
    //   type: Object,
    //   default: null
    // },
    format: {
      type: String,
      default: "image/png"
    },
    width: {
      type: Number,
      default: 512
    },
    height: {
      type: Number,
      default: 512
    }
  },
  watch: {
    layers(next) {
      if (this.initial) return;
      this.changelayers();
      /* if (next !== "") {
        this.mapSource.tiles = [this._url + "&layers=" + next];
      } else {
        this.mapSource.tiles = [this._url];
      } */
    }
  },
  methods: {
    changelayers() {
      this.$_init();
      this.mapSource.tiles = [this._url];
    },
    $_init() {
      if (this.url) {
        let url = this.url;
        if (url.indexOf("?") === -1) {
          url += "?";
          url += "bbox={bbox}";
        } else if (url.indexOf("bbox") === -1) {
          url += "&bbox={bbox}";
        }
        this._url = url;
        return;
      }
      let domain = this.domain;
      if (!domain) {
        domain = this.protocol + "://" + this.ip + ":" + this.port;
      }
      let tempUrl =
        domain +
        "/igs/rest/ogc/" +
        this.serverType +
        "/" +
        this.serverName +
        "/WMSServer";
      let partUrl = this.$_initAllRequestParams().join("&");
      this._url = encodeURI(tempUrl + "?" + partUrl) + "&bbox={bbox}";
    },
    $_initAllRequestParams() {
      let params = [];
      params.push("service=WMS");
      params.push("request=GetMap");
      params.push("version=" + this.version);
      params.push("layers=" + this.layers);
      params.push("format=" + this.format);
      params.push("width=" + this.width);
      params.push("height=" + this.height);
      if (this.version === "1.1.1") {
        params.push("srs=" + this.crs);
      } else if (this.version === "1.3.0") {
        params.push("crs=" + this.crs);
      }
      // if (this.styles) {
      //   params.push("styles=" + JSON.stringify(this.styles));
      // } else {
      params.push("styles=");
      //}
      return params;
    },
    $_deferredMount() {
      this.$_init();
      let source = {
        type: "raster",
        tiles: [this._url],
        tileSize: this.tileSize,
        ...this.source
      };

      this.map.on("dataloading", this.$_watchSourceLoading);
      try {
        this.map.addSource(this.sourceId, source);
      } catch (err) {
        if (this.replaceSource) {
          this.map.removeSource(this.sourceId);
          this.map.addSource(this.sourceId, source);
        }
      }
      this.$_addLayer();
      this.$_bindLayerEvents(layerEvents);
      this.map.off("dataloading", this.$_watchSourceLoading);
      this.initial = false;
    }
  }
};
